# Copyright 2025 Snowflake Inc.
# SPDX-License-Identifier: Apache-2.0

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os.path
import tempfile

import pytest

from snowflake.snowpark_checkpoints_configuration.checkpoint_metadata import (
    CheckpointMetadata,
)
from snowflake.snowpark_checkpoints_configuration.model.checkpoints import (
    Checkpoint,
    Checkpoints,
    Pipeline,
)
from snowflake.snowpark_checkpoints_configuration.singleton import Singleton


@pytest.fixture(autouse=True)
def singleton():
    Singleton._instances = {}


LOGGER_NAME = "snowflake.snowpark_checkpoints_configuration.checkpoint_metadata"


def test_checkpoint_metadata_loading(caplog: pytest.LogCaptureFixture):
    caplog.set_level(level=logging.INFO, logger=LOGGER_NAME)
    path = os.path.dirname(os.path.abspath(__file__))
    path = os.path.join(path, "valid_checkpoint")
    metadata = CheckpointMetadata(path)
    expected_checkpoint_1 = Checkpoint(
        name="demo_initial_creation_checkpoint",
        df="df",
        mode=1,
        function="",
        file="demo_pyspark_pipeline.py",
        location=1,
        enabled=True,
    )
    expected_checkpoint_2 = Checkpoint(
        name="demo_pyspark_pipeline_df1_123HDK",
        df="df1",
        mode=1,
        sample=0.5,
        file="demo_pyspark_pipeline.py",
        location=1,
        enabled=True,
    )
    expected_pipeline = Pipeline(
        entry_point="demo_pyspark_pipeline.py",
        checkpoints=[expected_checkpoint_1, expected_checkpoint_2],
    )

    expected_checkpoints = Checkpoints(type="Collection", pipelines=[expected_pipeline])
    assert metadata.checkpoint_model == expected_checkpoints
    assert "Successfully read and validated checkpoints file" in caplog.text


def test_checkpoint_metadata_loading_no_file(caplog: pytest.LogCaptureFixture):
    caplog.set_level(level=logging.WARNING, logger=LOGGER_NAME)
    path = tempfile.gettempdir()
    metadata = CheckpointMetadata(path)
    expected_checkpoints = Checkpoints(type="", pipelines=[])
    assert metadata.checkpoint_model == expected_checkpoints
    assert "Checkpoints file not found" in caplog.text


def test_checkpoint_metadata_loading_invalid_file(caplog: pytest.LogCaptureFixture):
    caplog.set_level(level=logging.ERROR, logger=LOGGER_NAME)
    path = os.path.dirname(os.path.abspath(__file__))
    path = os.path.join(path, "invalid_checkpoint")
    checkpoint_file_name = os.path.join(path, "checkpoints.json")
    expected_error_msg = f"An error occurred while reading the checkpoints file: '{checkpoint_file_name}'"
    with pytest.raises(Exception) as ex_info:
        CheckpointMetadata(path)
    assert str(ex_info.value).startswith(expected_error_msg)
    assert expected_error_msg in caplog.text
    assert any(
        record.exc_info is not None for record in caplog.records
    ), "Log message does not contain the exception details, but it should"


def test_checkpoint_metadata_get_checkpoint_exist():
    path = os.path.dirname(os.path.abspath(__file__))
    path = os.path.join(path, "valid_checkpoint")
    metadata = CheckpointMetadata(path)
    checkpoint = metadata.get_checkpoint("demo_initial_creation_checkpoint")
    expected_checkpoint = Checkpoint(
        name="demo_initial_creation_checkpoint",
        df="df",
        mode=1,
        function="",
        file="demo_pyspark_pipeline.py",
        location=1,
        enabled=True,
    )
    assert checkpoint == expected_checkpoint


def test_checkpoint_metadata_get_checkpoint_not_exist():
    path = os.path.dirname(os.path.abspath(__file__))
    path = os.path.join(path, "valid_checkpoint")
    metadata = CheckpointMetadata(path)
    checkpoint = metadata.get_checkpoint("not-exist-checkpoint")
    expected_checkpoint = Checkpoint(name="not-exist-checkpoint", enabled=False)
    assert checkpoint == expected_checkpoint


def test_checkpoint_metadata_get_checkpoint_no_file():
    path = tempfile.gettempdir()
    metadata = CheckpointMetadata(path)
    checkpoint = metadata.get_checkpoint("not-exist-checkpoint")
    expected_checkpoint = Checkpoint(name="not-exist-checkpoint", enabled=True)
    assert checkpoint == expected_checkpoint



# Copyright 2025 Snowflake Inc.
# SPDX-License-Identifier: Apache-2.0

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import pytest

from snowflake.snowpark_checkpoints_configuration.model.checkpoints import (
    Checkpoint,
    Checkpoints,
    Pipeline,
)


LOGGER_NAME = "snowflake.snowpark_checkpoints_configuration.model.checkpoints"


def test_add_checkpoint(caplog: pytest.LogCaptureFixture):
    caplog.set_level(level=logging.DEBUG, logger=LOGGER_NAME)
    checkpoints = Checkpoints(type="Collection", pipelines=[])
    normalized_checkpoint_name = "checkpoint_name"
    new_checkpoint = Checkpoint(
        name="checkpoint-name",
        df="df",
        mode=1,
        function="",
        file="file",
        location=1,
        enabled=False,
    )
    checkpoints.add_checkpoint(new_checkpoint)
    assert checkpoints.get_check_point(normalized_checkpoint_name) == new_checkpoint
    assert (
        f"Checkpoint '{normalized_checkpoint_name}' added successfully" in caplog.text
    )


def test_add_checkpoint_with_same_name():
    checkpoints = Checkpoints(type="Collection", pipelines=[])
    new_checkpoint = Checkpoint(
        name="checkpoint-name",
        df="df",
        mode=1,
        function="",
        file="file",
        location=1,
        enabled=False,
    )
    checkpoints.add_checkpoint(new_checkpoint)
    new_checkpoint_2 = Checkpoint(
        name="checkpoint_name",
        df="df2",
        mode=1,
        function="",
        file="file",
        location=1,
        enabled=True,
    )
    checkpoints.add_checkpoint(new_checkpoint_2)

    assert checkpoints.get_check_point("checkpoint_name") == new_checkpoint_2


def test_get_checkpoint_existing():
    new_checkpoint = Checkpoint(
        name="checkpoint-name",
        df="df",
        mode=1,
        function="",
        file="file",
        location=1,
        enabled=False,
    )
    new_pipeline = Pipeline(entry_point="entry-point", checkpoints=[new_checkpoint])
    checkpoints = Checkpoints(type="Collection", pipelines=[new_pipeline])

    expected_checkpoint = Checkpoint(
        name="checkpoint_name",
        df="df",
        mode=1,
        function="",
        file="file",
        location=1,
        enabled=False,
    )

    assert checkpoints.get_check_point("checkpoint_name") == expected_checkpoint


def test_get_checkpoint_non_existing_empty_dict(caplog: pytest.LogCaptureFixture):
    caplog.set_level(level=logging.DEBUG, logger=LOGGER_NAME)
    checkpoints = Checkpoints(type="Collection", pipelines=[])
    checkpoint_name = "checkpoint-name-2"
    checkpoint = checkpoints.get_check_point(checkpoint_name)
    assert checkpoint == Checkpoint(name=checkpoint_name, enabled=True)
    assert "creating a new enabled checkpoint" in caplog.text


def test_get_checkpoint_no_existing_non_empty_dict(caplog: pytest.LogCaptureFixture):
    caplog.set_level(level=logging.INFO, logger=LOGGER_NAME)
    checkpoint = Checkpoint(
        name="checkpoint_name_1",
        df="df",
        mode=1,
        function="",
        file="file",
        location=1,
        enabled=False,
    )
    pipeline = Pipeline(entry_point="entry-point", checkpoints=[checkpoint])
    checkpoints = Checkpoints(type="Collection", pipelines=[pipeline])
    checkpoint_name = "checkpoint_name_2"
    checkpoint = checkpoints.get_check_point(checkpoint_name)

    assert checkpoint == Checkpoint(name=checkpoint_name, enabled=False)
    assert "creating a new disabled checkpoint" in caplog.text
